@@ -190,6 +190,10 @@ class DynamoHandler(BaseResponse):
             er = "com.amazonaws.dynamodb.v20111205#ResourceNotFoundException"
             return self.error(er, "Requested resource not found")
 
+    def describe_endpoints(self):
+        response = {"Endpoints": self.dynamodb_backend.describe_endpoints()}
+        return dynamo_json_dump(response)
+
     def tag_resource(self):
         table_arn = self.body["ResourceArn"]
         tags = self.body["Tags"]
@@ -751,8 +755,12 @@ class DynamoHandler(BaseResponse):
         return_values = self.body.get("ReturnValues", "NONE")
         update_expression = self.body.get("UpdateExpression", "").strip()
         attribute_updates = self.body.get("AttributeUpdates")
-        expression_attribute_names = self.body.get("ExpressionAttributeNames", {})
-        expression_attribute_values = self.body.get("ExpressionAttributeValues", {})
+        if update_expression and attribute_updates:
+            er = "com.amazonaws.dynamodb.v20111205#ValidationException"
+            return self.error(
+                er,
+                "Can not use both expression and non-expression parameters in the same request: Non-expression parameters: {AttributeUpdates} Expression parameters: {UpdateExpression}",
+            )
         # We need to copy the item in order to avoid it being modified by the update_item operation
         existing_item = copy.deepcopy(self.dynamodb_backend.get_item(name, key))
         if existing_item:
