@@ -3,7 +3,6 @@ from __future__ import unicode_literals
 from moto.core.responses import BaseResponse
 
 from .models import dynamodbstreams_backends
-from six import string_types
 
 
 class DynamoDBStreamsHandler(BaseResponse):
@@ -25,7 +24,7 @@ class DynamoDBStreamsHandler(BaseResponse):
         shard_iterator_type = self._get_param("ShardIteratorType")
         sequence_number = self._get_param("SequenceNumber")
         # according to documentation sequence_number param should be string
-        if isinstance(sequence_number, string_types):
+        if isinstance(sequence_number, str):
             sequence_number = int(sequence_number)
 
         return self.backend.get_shard_iterator(
