@@ -3,8 +3,8 @@ from .responses import KinesisResponse
 
 url_bases = [
     # Need to avoid conflicting with kinesisvideo
-    r"https?://kinesis\.(.+).amazonaws.com",
-    "https?://firehose.(.+).amazonaws.com",
+    r"https?://kinesis\.(.+)\.amazonaws\.com",
+    r"https?://firehose\.(.+)\.amazonaws\.com",
 ]
 
 url_paths = {"{0}/$": KinesisResponse.dispatch}
